from __future__ import absolute_import
import os
import sys
import json
import configparser

from blendedUx.blended_flask.blended_hostlib.swagger_client.apis.packages_api import PackagesApi
from blendedUx.blended_flask.blended_hostlib.swagger_client.apis.accounts_api import AccountsApi
from blendedUx.blended_flask.blended_hostlib.swagger_client.rest import ApiException

from blendedUx.blended_flask.blended_hostlib.exceptions import BlendedException, \
    PackageNameExistsException, AccountActivationException

import urllib3
from .settings import DIRECTORY, USER_SESSION as user_rc_file
# urllib3.disable_warnings()
# import logging


class Network(object):
    """
    class to interact with client api.
    """
    def __init__(self, *args, **kwargs):
        """
        """
        self.account_client = AccountsApi()
        self.package_client = PackagesApi()
        self.session_key = kwargs.get('session_key', None)
        if not self.session_key:
            self.session_key, self.user_pk = self.get_sessionkey()

    def login(self, username, password, active_account=''):
        """
        """
        body = {'user_name': username, 'password': password, 'active_account':active_account}
        try:
            response = self.account_client.login(body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                if error_dict['status_code'] == 5018:
                    raise AccountActivationException(error_dict['errors'])
                else:
                    raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def logout(self):
        """
        """
        sessionkey = self.session_key
        body = {'sessionkey': sessionkey}
        try:
            response = self.account_client.logout(body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                if error_dict['status_code'] == 5018:
                    raise AccountActivationException(error_dict['errors'])
                else:
                    raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def create_account(self, body):
        """
        """
        try:
            response = self.account_client.create_account(body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def update_account(self, pk, body):
        """
        """
        sessionkey=self.session_key
        try:
            response = self.account_client.update_account(pk, body, sessionkey=sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_current_account(self, pk):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.account_client.get_current_account(pk, sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_account_list(self, pk):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.account_client.get_account_list(sessionkey, pk)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_account_users(self, slug):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.account_client.get_account_users(slug, sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def remove_account(self, slug, account_slug,):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.account_client.remove_account(slug, account_slug, sessionkey)
            response = "TODO"
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def resend_account_verification_email(self, body):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.account_client.resend_account_verification_email(sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                if error_dict['status_code'] == 5086:
                    raise AccountActivationException(error_dict['message'])
                elif error_dict['status_code'] == 4035:
                    raise AccountActivationException(error_dict['errors']['email'][0])
                else:
                    raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_sessionkey(self):
        """
        Read session key from user config file
        """
        config = configparser.ConfigParser()
        working_dir = DIRECTORY
        user_rc_path = os.path.join(working_dir, user_rc_file)
        try:
            config.read(user_rc_path)
            session_key = config.get('USER', 'sessionkey')
            user_pk = config.get('USER', 'user_pk')
        except (configparser.NoSectionError, configparser.NoOptionError, OSError, IOError):
            session_key = None
            user_pk = None
        return session_key, user_pk

    def set_sessionkey(self, session_key, user_pk):
        """
        Stores sessionkey in filesystem.
        """
        self.session_key = session_key
        self.user_pk = user_pk
        config = configparser.ConfigParser()
        working_dir = DIRECTORY
        user_rc_path = os.path.join(working_dir, user_rc_file)
        config.read(user_rc_path)
        try:
            config.set('USER', 'sessionkey', str(session_key))
            config.set('USER', 'user_pk', str(user_pk))
        except (configparser.NoSectionError, configparser.NoOptionError):
            config.add_section('USER')
            config.set('USER', 'sessionkey', str(session_key))
            config.set('USER', 'user_pk', str(user_pk))

        with open(user_rc_path, 'w') as config_file:
            config.write(config_file)

    def set_user_pk(self, user_pk):
        """
        Stores user_pk in filesystem.
        """
        self.user_pk = user_pk
        config = configparser.ConfigParser()
        working_dir = DIRECTORY
        user_rc_path = os.path.join(working_dir, user_rc_file)
        config.read(user_rc_path)
        try:
            config.set('USER', 'user_pk', str(user_pk))
        except (configparser.NoSectionError, configparser.NoOptionError):
            config.add_section('USER')
            config.set('USER', 'user_pk', str(user_pk))
        with open(user_rc_path, 'w') as config_file:
            config.write(config_file)

    def get_user_pk(self):
        """
        Read session key from user config file
        """
        config = configparser.ConfigParser()
        working_dir = DIRECTORY
        user_rc_path = os.path.join(working_dir, user_rc_file)
        try:
            config.read(user_rc_path)
            user_pk = config.get('USER', 'user_pk')
        except (configparser.NoSectionError, configparser.NoOptionError):
            user_pk = 'anonymous'
            # raise BlendedException('user is not created yet')
        return user_pk

    def invite_user(self, slug, body):
        """
        :param slug:
        :param body:
        :return:
        """
        sessionkey = self.session_key
        try:
            response = self.account_client.invite_user(slug, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def change_account_user_privilege(self, slug, user_slug, body):
        """
        :param slug:
        :param body:
        :return:
        """
        sessionkey = self.session_key
        try:
            response = self.account_client.change_account_user_privilege(slug, user_slug, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

        

    def add_account_user(self, slug, body):
        """
        :param slug:
        :param body:
        :return:
        """
        sessionkey = self.session_key
        try:
            response = self.account_client.add_account_user(slug, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def set_current_account(self, body):
        """
        :param body:
        :return:
        """
        slug = self.user_pk
        sessionkey = self.session_key
        try:
            response = self.account_client.set_current_account(slug, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def create_package(self, account_slug, body):
        """
        """
        sessionkey = self.session_key
        # if description:
        #     body = {'name': package_name,
        #             'type': package_type,
        #             'description': description}
        # else:
        #     body = {'name': package_name,
        #             'type': package_type
        #            }
        try:
            response = self.package_client.create_package(account_slug, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def acquire_package(self, account_slug, pk, body):
        """
        method interacts with the client api for getting a
        package from hub
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.acquire_package(account_slug, pk, sessionkey, body)
        except ApiException as exc:
            try:
                # with open('shareerror.html', 'w') as filename:
                #     filename.write(exc.body)
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                if ((error_dict['status_code'] == 5999) and
                        (error_dict['message'] == 'package with given slug already exists in account')):
                    raise PackageNameExistsException()
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_package_details(self, account_slug, pk):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.get_package_details(account_slug, pk, sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_version_details(self, account_slug, pk, label):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.get_version_details(account_slug, pk, label, sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_detail_document(self, account_slug, slug, name):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.get_detail_document(account_slug, slug, name, sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def snapshot(self, account_slug, pk, body):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.snapshot(account_slug, pk, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def set_canonical(self, account_slug, pk, label, body):
        """
        """
        body['label'] = label
        sessionkey = self.session_key
        try:
            response = self.package_client.canonical_set(account_slug, pk, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def create_draft(self, account_slug, pk, body):
        """
        method interacts with the client api for saving a
        package draft to the hub
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.create_draft(account_slug, pk, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                # {'status_code': 5059, 'message': 'Draft has been changed. Please pull it first'}
                # if error_dict['status_code'] == 5059:
                #     raise BlendedException('Package with given slug already exists in account.')
                # else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def update_draft(self, account_slug, pk, body):
        """
        method interacts with the client api for saving a
        package draft to the hub
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.update_draft(account_slug, pk, sessionkey, body)
        except ApiException as exc:
            try:
                with open('shareerror.html', 'w') as filename:
                    filename.write(exc.body)
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def package_list(self, account_slug, slug=None, share=None, transfer_to_me=None,
                     transfer_by_me=None, organization=None, published=None, purchased=None):
        """
        """
        body = {'slug': slug}
        sessionkey = self.session_key
        try:
            if share:
                response = self.package_client.get_packages(account_slug, sessionkey, share='1')
            elif transfer_to_me:
                response = self.package_client.get_packages(account_slug, sessionkey, transfer_to_me='1')
            elif transfer_by_me:
                response = self.package_client.get_packages(account_slug, sessionkey, transfer_by_me='1')
            elif organization:
                response = self.package_client.get_packages(account_slug, sessionkey, org_packages='1')
            elif purchased:
                response = self.package_client.get_packages(account_slug, sessionkey, purchased='1')
            elif published:
                response = self.package_client.get_packages(account_slug, sessionkey, published='1')
            else:
                response = self.package_client.get_packages(account_slug, sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_versions_list(self, account_slug, slug):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.get_versions(account_slug, slug, sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def package_accounts_list(self, account_slug, package_name):  # package_accounts_list(self, account_slug, slug, package_name)
        """
        :param slug:
        :param package_name:
        :return:
        """
        body = {'slug': account_slug, 'package_name': package_name}
        sessionkey = self.session_key
        try:
            response = self.package_client.package_account_list(account_slug, package_name, sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_dependencies(self, account_slug, slug):
        """
        :param slug:
        :return:
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.get_dependencies(account_slug, slug, sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def update_package(self, account_slug, pk, body):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.update_package(account_slug, pk, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def add_license(self, account_slug, slug, label, body):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.add_license(account_slug, slug, label, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                if error_dict['status_code'] == 9999:
                    raise BlendedException(error_dict['message'])
                else:
                    raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def remove_license(self, pk, name):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.remove_license(sessionkey, pk, name)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def download(self, account_slug, pk, label, as_hashes='0'):
        """
        """
        sessionkey = self.session_key
        try:
            if sessionkey:
                response = self.package_client.download(account_slug, pk, label, sessionkey=sessionkey, as_hashes=as_hashes)
            else:
                response = self.package_client.download(account_slug, pk, label, as_hashes=as_hashes)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def download_canonical(self, account_slug, pk):  # download_canonical(self, account_slug, pk, as_hashes='0')
        """
        """
        sessionkey = self.session_key
        try:
            if sessionkey:
                response = self.package_client.download_canonical(account_slug, pk, sessionkey=sessionkey)
            else:
                response = self.package_client.download_canonical(account_slug, pk)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def download_draft(self, account_slug, pk, as_hashes='0'):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.download_draft(account_slug, pk, sessionkey, as_hashes=as_hashes)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_canonical(self, account_slug, package_slug):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.get_canonical(account_slug, package_slug, sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def upload_media(self, account_name, pk, hash_value, **kwargs):
        """
        """
        try:
            if(not kwargs.get('token', None)):
                # if kwargs is having token and it's value is NOne then remove token from kwargs
                del kwargs['token']
            response = self.package_client.upload_media(account_name, pk, self.session_key, hash_value, **kwargs)
            return response
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")

    def get_package_initial(self):
        """
        """
        try:
            response = self.package_client.get_package_initial()
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def get_publication(self, account_slug, pk):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.get_publication(account_slug, pk, sessionkey=sessionkey)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def update_publication(self, account_slug, pk, body, commit='1'):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.update_publication(account_slug, pk, body, sessionkey=sessionkey, commit=commit)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def validate_publication(self, account_slug, pk, body):
        """
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.validate_publication(account_slug, pk, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def revoke_acquired_package(self, account_slug, pk, body):
        """
        method interacts with the client api for getting a
        package from hub
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.revoke_acquired_package(account_slug, pk, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict) 
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response


    def accept_acquisition(self, account_slug, pk, body):
        """
        method interacts with the client api for getting a
        package from hub
        """
        sessionkey = self.session_key
        
        try:
            response = self.package_client.accept_acquisition(account_slug, pk, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict) 
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response


    def get_acquired_package_account_list(self, account_slug, pk):
        """
        method interacts with the client api for getting a
        package from hub
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.get_acquired_package_account_list(account_slug, pk, sessionkey, share='1')
        except ApiException as exc:
            try:
                 error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict)
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response

    def reject_acquisition_invite(self, account_slug, rejected_account, pk, body):
        """
        method interacts with the client api to reject package acquisition.
        """
        sessionkey = self.session_key
        try:
            response = self.package_client.reject_acquisition_invite(rejected_account, pk, sessionkey, body)
        except ApiException as exc:
            try:
                error_dict = json.loads(exc.body)
            except ValueError:
                raise BlendedException("Something went wrong on the Server")
            else:
                raise BlendedException(error_dict) 
        except urllib3.exceptions.MaxRetryError:
            raise BlendedException("\nFailed to establish connection. Please try again.")
        return response 


# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class AllLicensePrice(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, license_name=None, license_type=None, price=None, total_price=None, listed=None, get_or_purchased=None):
        """
        AllLicensePrice - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'license_name': 'str',
            'license_type': 'str',
            'price': 'float',
            'total_price': 'float',
            'listed': 'bool',
            'get_or_purchased': 'str'
        }

        self.attribute_map = {
            'license_name': 'license_name',
            'license_type': 'license_type',
            'price': 'price',
            'total_price': 'total_price',
            'listed': 'listed',
            'get_or_purchased': 'get_or_purchased'
        }

        self._license_name = license_name
        self._license_type = license_type
        self._price = price
        self._total_price = total_price
        self._listed = listed
        self._get_or_purchased = get_or_purchased

    @property
    def license_name(self):
        """
        Gets the license_name of this AllLicensePrice.
        Name of License of pacakge

        :return: The license_name of this AllLicensePrice.
        :rtype: str
        """
        return self._license_name

    @license_name.setter
    def license_name(self, license_name):
        """
        Sets the license_name of this AllLicensePrice.
        Name of License of pacakge

        :param license_name: The license_name of this AllLicensePrice.
        :type: str
        """

        self._license_name = license_name

    @property
    def license_type(self):
        """
        Gets the license_type of this AllLicensePrice.
        Type of License of pacakge. Ex: MIT, BCL, etc.

        :return: The license_type of this AllLicensePrice.
        :rtype: str
        """
        return self._license_type

    @license_type.setter
    def license_type(self, license_type):
        """
        Sets the license_type of this AllLicensePrice.
        Type of License of pacakge. Ex: MIT, BCL, etc.

        :param license_type: The license_type of this AllLicensePrice.
        :type: str
        """

        self._license_type = license_type

    @property
    def price(self):
        """
        Gets the price of this AllLicensePrice.
        price of license

        :return: The price of this AllLicensePrice.
        :rtype: float
        """
        return self._price

    @price.setter
    def price(self, price):
        """
        Sets the price of this AllLicensePrice.
        price of license

        :param price: The price of this AllLicensePrice.
        :type: float
        """

        self._price = price

    @property
    def total_price(self):
        """
        Gets the total_price of this AllLicensePrice.
        total price of license including bundles

        :return: The total_price of this AllLicensePrice.
        :rtype: float
        """
        return self._total_price

    @total_price.setter
    def total_price(self, total_price):
        """
        Sets the total_price of this AllLicensePrice.
        total price of license including bundles

        :param total_price: The total_price of this AllLicensePrice.
        :type: float
        """

        self._total_price = total_price

    @property
    def listed(self):
        """
        Gets the listed of this AllLicensePrice.
        Allow listed flag of license

        :return: The listed of this AllLicensePrice.
        :rtype: bool
        """
        return self._listed

    @listed.setter
    def listed(self, listed):
        """
        Sets the listed of this AllLicensePrice.
        Allow listed flag of license

        :param listed: The listed of this AllLicensePrice.
        :type: bool
        """

        self._listed = listed

    @property
    def get_or_purchased(self):
        """
        Gets the get_or_purchased of this AllLicensePrice.
        Name of License of pacakge

        :return: The get_or_purchased of this AllLicensePrice.
        :rtype: str
        """
        return self._get_or_purchased

    @get_or_purchased.setter
    def get_or_purchased(self, get_or_purchased):
        """
        Sets the get_or_purchased of this AllLicensePrice.
        Name of License of pacakge

        :param get_or_purchased: The get_or_purchased of this AllLicensePrice.
        :type: str
        """

        self._get_or_purchased = get_or_purchased

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



import re
import os
import sys
import zlib
import base64
import hashlib
import json

from blendedUx.blended_flask.blended_hostlib.exceptions import BlendedException
from .settings import *
user_rc_file = USER_RC
DEFAULT_CHUNK_SIZE = 64 * 2 ** 10

alias = DEFAULT_ALIAS
package_list_rc_file = PACKAGE_LIST_RC

class IntermediaryNode(object):
    """
    """
    _hash = ''

    def __init__(self, location, content=None, name=None, hash=None,
                 as_hashes=False, removed=False, empty_dir=False,
                 rem_loc_dir=False, rem_from_hub_dir=False,
                 add_emp_dir_on_hub=False, path_validate_errors=None):
        """
        """
        self.location = location
        self.empty_dir = empty_dir
        self.rem_loc_dir = rem_loc_dir
        self.rem_from_hub_dir = rem_from_hub_dir
        self.add_emp_dir_on_hub = add_emp_dir_on_hub
        self.path_validate_errors = path_validate_errors or []

        if name:
            self.name = name
        else:
            self.name = location.split(os.sep, -1)[-1]
        
        if content or content=='':
            self.content = content
        elif as_hashes:
            self.content = None
        else:
            if empty_dir:
                self.content = []
            else:
                validation_error = self.validate_location(location)
                if validation_error:
                    self.path_validate_errors.append(validation_error)
                self.content = self.get_content(location)
        if hash:
            self.hash = hash
        elif removed:
            self.hash = ''
        else:
            self.hash = self.get_hash()
       
    def validate_location(self, file_location):
        """
        """
        filename, file_extension = os.path.splitext(file_location)
        rel_path = os.path.relpath(file_location)
        name = rel_path
        if os.sep == '\\':
            pattern = re.compile('[@!#$%^&*()\[\]<=>?\|}{~;,`\'\"\s+\t+]')
        else:
            pattern = re.compile('[@!#$%^&*()\[\]<=>?\|\\\\}{~:;,`\'\"\s+\t+]')
        error = 'Please enter a valid name. (White-spaces and special characters are not allowed except - and _).'
        if pattern.search(filename):
            return {'file': name, 'error' : error  }
        if os.path.isdir(file_location):
            #if '.' in name:
            #    return {'file': name, 'error' : error  }
            pass
        elif file_extension.lower() in ALLOWED_IMAGE_TYPES:
            pass
        elif file_extension.lower() == '.json':
            pass
        elif file_extension.lower() in RESTRICT_IMAGE_TYPE:
            error = 'Please enter a valid extension. (The allowed image extensions are .jpg, .jpeg, .png).'
            return {'file': name, 'error' : error  }
        elif file_extension.lower() not in ALLOWED_FILE_TYPES:
            error = 'Please enter a valid extension. (The allowed file extensions are .html, .css, .bd, .js, .json, .svg, .txt).'
            return {'file': name, 'error' : error  }
        return None

    def get_hash(self):
        """
        :return:
        """
        if self._hash:
            return self._hash
        else:
            self._hash = self.generate_hash()
            return self._hash

    def get_content(self, location):
        """
        :return:
        """
        return


class Directory(IntermediaryNode):
    """
    """
    def get_content(self, location):
        """
        """
        error_content = []
        content = []
        if (os.path.exists(location)) and (not self.empty_dir):
            directory_content = os.listdir(location)
            for each_file in directory_content:
                file_location = os.path.join(location, each_file)
                filename, file_extension = os.path.splitext(file_location)
                if(each_file.endswith(IGNORE_EXTENSIONS)):
                    continue
                if (each_file.startswith(IGNORE_PREFIXES)) and (each_file != '_package.json') and (each_file != '_index.json'):
                    continue
                content.append(self.get_content_at_path(file_location))
            return content

    def get_content_at_path(self, file_location):
        """
        """
        filename, file_extension = os.path.splitext(file_location)
        name  = os.path.basename(file_location)
        rel_path = os.path.relpath(file_location)
        nodeData = None
        if os.path.isdir(file_location):
            nodeData = Directory(file_location)
        elif file_extension.lower() in ALLOWED_IMAGE_TYPES:
            nodeData = BinaryFile(file_location)
        elif file_extension.lower() == '.json':
            nodeData = JSONFile(file_location)
        else:
            try:
                nodeData = TextFile(file_location)
            except: 
                nodeData = BinaryFile(file_location)
        if nodeData.path_validate_errors:
            self.path_validate_errors.extend(nodeData.path_validate_errors)
        return nodeData
    
    def generate_hash(self):
        """
        :return:
        """
        file_hash_list = []
        for content_file in self.content:
            file_hash_list.append('%s:%s' %
                                  (content_file.name, content_file.get_hash()))
        final_hash = hashlib.md5('::'.join(sorted(file_hash_list)).encode('utf-8')).hexdigest()
        return final_hash


class TextFile(IntermediaryNode):
    """
    """
    def get_content(self, file_location):
        """
        """
        with open(file_location, 'r', encoding='utf-8') as content_file:
            file_data = content_file.read()
        return file_data

    def generate_hash(self):
        """
        :return:
        """
        try:
            return hashlib.md5(self.content.encode('utf-8')).hexdigest()
        except UnicodeDecodeError:
            return hashlib.md5(self.content).hexdigest()


class BinaryFile(IntermediaryNode):
    """
    """
    def get_content(self, file_location):
        """
        """
        return self._get_media(file_location)

    def generate_hash(self):
        """
        :return:
        """
        # return self._compute_hash(self.content)
        hash = self._compute_hash(self.content)
        self.content.close()
        return hash

    def _get_media(self, image_path):
        """
        """
        image_path = os.path.join(image_path)
        image = open(image_path, 'rb')
        return image

    def _compute_hash(self, content):
        """
        :return:
        """
        hasher = hashlib.sha1()
        cursor = content.tell()
        content.seek(0)
        try:
            while True:
                data = content.read(DEFAULT_CHUNK_SIZE)
                if not data:
                    break
                hasher.update(data)
            return hasher.hexdigest()
        finally:
            content.seek(cursor)


class JSONFile(IntermediaryNode):
    """
    """
    def get_content(self, file_location):
        """
        """
        with open(file_location, 'r') as content_file:
            content_file_data = content_file.read()
            try:
                if not content_file_data:
                    content = ''
                else:
                    content = json.loads(content_file_data)
            except ValueError:
                raise BlendedException("Invalid JSON content in File %s" % self.name)
            return self._JSON_as_string(content)

    def generate_hash(self):
        """
        :return:
        """
        return hashlib.md5(self.content.encode('utf-8')).hexdigest()

    def _JSON_as_string(self, content):
        """
        :param content:
        :return:
        """
        return json.dumps(content, separators=(',', ':'), sort_keys=True)




# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class PublicationVersion(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, label=None, canonical=None, bundle_packages=None, created_date=None, bundles_price=None, is_set_canonical=None, validation_status=None):
        """
        PublicationVersion - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'label': 'str',
            'canonical': 'bool',
            'bundle_packages': 'list[BundledPackages]',
            'created_date': 'str',
            'bundles_price': 'float',
            'is_set_canonical': 'int',
            'validation_status': 'str'
        }

        self.attribute_map = {
            'label': 'label',
            'canonical': 'canonical',
            'bundle_packages': 'bundle_packages',
            'created_date': 'created_date',
            'bundles_price': 'bundles_price',
            'is_set_canonical': 'is_set_canonical',
            'validation_status': 'validation_status'
        }

        self._label = label
        self._canonical = canonical
        self._bundle_packages = bundle_packages
        self._created_date = created_date
        self._bundles_price = bundles_price
        self._is_set_canonical = is_set_canonical
        self._validation_status = validation_status

    @property
    def label(self):
        """
        Gets the label of this PublicationVersion.
        label of package version

        :return: The label of this PublicationVersion.
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """
        Sets the label of this PublicationVersion.
        label of package version

        :param label: The label of this PublicationVersion.
        :type: str
        """

        self._label = label

    @property
    def canonical(self):
        """
        Gets the canonical of this PublicationVersion.
        canonical boolean value

        :return: The canonical of this PublicationVersion.
        :rtype: bool
        """
        return self._canonical

    @canonical.setter
    def canonical(self, canonical):
        """
        Sets the canonical of this PublicationVersion.
        canonical boolean value

        :param canonical: The canonical of this PublicationVersion.
        :type: bool
        """

        self._canonical = canonical

    @property
    def bundle_packages(self):
        """
        Gets the bundle_packages of this PublicationVersion.
        bundle package of given version label

        :return: The bundle_packages of this PublicationVersion.
        :rtype: list[BundledPackages]
        """
        return self._bundle_packages

    @bundle_packages.setter
    def bundle_packages(self, bundle_packages):
        """
        Sets the bundle_packages of this PublicationVersion.
        bundle package of given version label

        :param bundle_packages: The bundle_packages of this PublicationVersion.
        :type: list[BundledPackages]
        """

        self._bundle_packages = bundle_packages

    @property
    def created_date(self):
        """
        Gets the created_date of this PublicationVersion.
        version create date

        :return: The created_date of this PublicationVersion.
        :rtype: str
        """
        return self._created_date

    @created_date.setter
    def created_date(self, created_date):
        """
        Sets the created_date of this PublicationVersion.
        version create date

        :param created_date: The created_date of this PublicationVersion.
        :type: str
        """

        self._created_date = created_date

    @property
    def bundles_price(self):
        """
        Gets the bundles_price of this PublicationVersion.
        total price of bundled packages

        :return: The bundles_price of this PublicationVersion.
        :rtype: float
        """
        return self._bundles_price

    @bundles_price.setter
    def bundles_price(self, bundles_price):
        """
        Sets the bundles_price of this PublicationVersion.
        total price of bundled packages

        :param bundles_price: The bundles_price of this PublicationVersion.
        :type: float
        """

        self._bundles_price = bundles_price

    @property
    def is_set_canonical(self):
        """
        Gets the is_set_canonical of this PublicationVersion.
        flag to know if canonical version was set by user

        :return: The is_set_canonical of this PublicationVersion.
        :rtype: int
        """
        return self._is_set_canonical

    @is_set_canonical.setter
    def is_set_canonical(self, is_set_canonical):
        """
        Sets the is_set_canonical of this PublicationVersion.
        flag to know if canonical version was set by user

        :param is_set_canonical: The is_set_canonical of this PublicationVersion.
        :type: int
        """

        self._is_set_canonical = is_set_canonical

    @property
    def validation_status(self):
        """
        Gets the validation_status of this PublicationVersion.
        current validation status based on badge results and validation plans passing level

        :return: The validation_status of this PublicationVersion.
        :rtype: str
        """
        return self._validation_status

    @validation_status.setter
    def validation_status(self, validation_status):
        """
        Sets the validation_status of this PublicationVersion.
        current validation status based on badge results and validation plans passing level

        :param validation_status: The validation_status of this PublicationVersion.
        :type: str
        """

        self._validation_status = validation_status

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class SessionData(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, session_key=None, slug=None, active_account=None):
        """
        SessionData - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'session_key': 'str',
            'slug': 'str',
            'active_account': 'str'
        }

        self.attribute_map = {
            'session_key': 'session_key',
            'slug': 'slug',
            'active_account': 'active_account'
        }

        self._session_key = session_key
        self._slug = slug
        self._active_account = active_account

    @property
    def session_key(self):
        """
        Gets the session_key of this SessionData.
        value used to manage user session.

        :return: The session_key of this SessionData.
        :rtype: str
        """
        return self._session_key

    @session_key.setter
    def session_key(self, session_key):
        """
        Sets the session_key of this SessionData.
        value used to manage user session.

        :param session_key: The session_key of this SessionData.
        :type: str
        """

        self._session_key = session_key

    @property
    def slug(self):
        """
        Gets the slug of this SessionData.
        unique identifier for the user.

        :return: The slug of this SessionData.
        :rtype: str
        """
        return self._slug

    @slug.setter
    def slug(self, slug):
        """
        Sets the slug of this SessionData.
        unique identifier for the user.

        :param slug: The slug of this SessionData.
        :type: str
        """

        self._slug = slug

    @property
    def active_account(self):
        """
        Gets the active_account of this SessionData.
        slug if you want to activate any of your organizations.

        :return: The active_account of this SessionData.
        :rtype: str
        """
        return self._active_account

    @active_account.setter
    def active_account(self, active_account):
        """
        Sets the active_account of this SessionData.
        slug if you want to activate any of your organizations.

        :param active_account: The active_account of this SessionData.
        :type: str
        """

        self._active_account = active_account

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class ValidatorPlan(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, href=None, pk=None, validations_completed=None, request_date=None, validators=None):
        """
        ValidatorPlan - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'href': 'str',
            'pk': 'int',
            'validations_completed': 'bool',
            'request_date': 'datetime',
            'validators': 'object'
        }

        self.attribute_map = {
            'href': 'href',
            'pk': 'pk',
            'validations_completed': 'validations_completed',
            'request_date': 'request_date',
            'validators': 'validators'
        }

        self._href = href
        self._pk = pk
        self._validations_completed = validations_completed
        self._request_date = request_date
        self._validators = validators

    @property
    def href(self):
        """
        Gets the href of this ValidatorPlan.
        current object reference url

        :return: The href of this ValidatorPlan.
        :rtype: str
        """
        return self._href

    @href.setter
    def href(self, href):
        """
        Sets the href of this ValidatorPlan.
        current object reference url

        :param href: The href of this ValidatorPlan.
        :type: str
        """

        self._href = href

    @property
    def pk(self):
        """
        Gets the pk of this ValidatorPlan.
        value used to uniquely identify validatorplan.

        :return: The pk of this ValidatorPlan.
        :rtype: int
        """
        return self._pk

    @pk.setter
    def pk(self, pk):
        """
        Sets the pk of this ValidatorPlan.
        value used to uniquely identify validatorplan.

        :param pk: The pk of this ValidatorPlan.
        :type: int
        """

        self._pk = pk

    @property
    def validations_completed(self):
        """
        Gets the validations_completed of this ValidatorPlan.
        validation is completed or not.

        :return: The validations_completed of this ValidatorPlan.
        :rtype: bool
        """
        return self._validations_completed

    @validations_completed.setter
    def validations_completed(self, validations_completed):
        """
        Sets the validations_completed of this ValidatorPlan.
        validation is completed or not.

        :param validations_completed: The validations_completed of this ValidatorPlan.
        :type: bool
        """

        self._validations_completed = validations_completed

    @property
    def request_date(self):
        """
        Gets the request_date of this ValidatorPlan.
        request date for validator plan.

        :return: The request_date of this ValidatorPlan.
        :rtype: datetime
        """
        return self._request_date

    @request_date.setter
    def request_date(self, request_date):
        """
        Sets the request_date of this ValidatorPlan.
        request date for validator plan.

        :param request_date: The request_date of this ValidatorPlan.
        :type: datetime
        """

        self._request_date = request_date

    @property
    def validators(self):
        """
        Gets the validators of this ValidatorPlan.
        validators attached with validatorplan.

        :return: The validators of this ValidatorPlan.
        :rtype: object
        """
        return self._validators

    @validators.setter
    def validators(self, validators):
        """
        Sets the validators of this ValidatorPlan.
        validators attached with validatorplan.

        :param validators: The validators of this ValidatorPlan.
        :type: object
        """

        self._validators = validators

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class Validator(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, href=None, status=None, description=None, framework=None, name=None, passed=None, score=None, error_messages=None, warning_messages=None):
        """
        Validator - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'href': 'str',
            'status': 'str',
            'description': 'str',
            'framework': 'str',
            'name': 'str',
            'passed': 'bool',
            'score': 'int',
            'error_messages': 'str',
            'warning_messages': 'str'
        }

        self.attribute_map = {
            'href': 'href',
            'status': 'status',
            'description': 'description',
            'framework': 'framework',
            'name': 'name',
            'passed': 'passed',
            'score': 'score',
            'error_messages': 'error_messages',
            'warning_messages': 'warning_messages'
        }

        self._href = href
        self._status = status
        self._description = description
        self._framework = framework
        self._name = name
        self._passed = passed
        self._score = score
        self._error_messages = error_messages
        self._warning_messages = warning_messages

    @property
    def href(self):
        """
        Gets the href of this Validator.
        current object reference url

        :return: The href of this Validator.
        :rtype: str
        """
        return self._href

    @href.setter
    def href(self, href):
        """
        Sets the href of this Validator.
        current object reference url

        :param href: The href of this Validator.
        :type: str
        """

        self._href = href

    @property
    def status(self):
        """
        Gets the status of this Validator.
        validator status.

        :return: The status of this Validator.
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """
        Sets the status of this Validator.
        validator status.

        :param status: The status of this Validator.
        :type: str
        """

        self._status = status

    @property
    def description(self):
        """
        Gets the description of this Validator.
        validator description.

        :return: The description of this Validator.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this Validator.
        validator description.

        :param description: The description of this Validator.
        :type: str
        """

        self._description = description

    @property
    def framework(self):
        """
        Gets the framework of this Validator.
        validator framework.

        :return: The framework of this Validator.
        :rtype: str
        """
        return self._framework

    @framework.setter
    def framework(self, framework):
        """
        Sets the framework of this Validator.
        validator framework.

        :param framework: The framework of this Validator.
        :type: str
        """

        self._framework = framework

    @property
    def name(self):
        """
        Gets the name of this Validator.
        validator name.

        :return: The name of this Validator.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this Validator.
        validator name.

        :param name: The name of this Validator.
        :type: str
        """

        self._name = name

    @property
    def passed(self):
        """
        Gets the passed of this Validator.
        validator is passed or not.

        :return: The passed of this Validator.
        :rtype: bool
        """
        return self._passed

    @passed.setter
    def passed(self, passed):
        """
        Sets the passed of this Validator.
        validator is passed or not.

        :param passed: The passed of this Validator.
        :type: bool
        """

        self._passed = passed

    @property
    def score(self):
        """
        Gets the score of this Validator.
        validator score after validation.

        :return: The score of this Validator.
        :rtype: int
        """
        return self._score

    @score.setter
    def score(self, score):
        """
        Sets the score of this Validator.
        validator score after validation.

        :param score: The score of this Validator.
        :type: int
        """

        self._score = score

    @property
    def error_messages(self):
        """
        Gets the error_messages of this Validator.
        Error messages after validator run.

        :return: The error_messages of this Validator.
        :rtype: str
        """
        return self._error_messages

    @error_messages.setter
    def error_messages(self, error_messages):
        """
        Sets the error_messages of this Validator.
        Error messages after validator run.

        :param error_messages: The error_messages of this Validator.
        :type: str
        """

        self._error_messages = error_messages

    @property
    def warning_messages(self):
        """
        Gets the warning_messages of this Validator.
        warning messages after validator run.

        :return: The warning_messages of this Validator.
        :rtype: str
        """
        return self._warning_messages

    @warning_messages.setter
    def warning_messages(self, warning_messages):
        """
        Sets the warning_messages of this Validator.
        warning messages after validator run.

        :param warning_messages: The warning_messages of this Validator.
        :type: str
        """

        self._warning_messages = warning_messages

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



from __future__ import absolute_import
import hashlib

def blended_hash(layout, prefix=None):
    if layout:
        try:
            hash_value = hashlib.md5(repr(layout).encode('utf-8')).hexdigest()
            if prefix:
                prefix = prefix.replace(" ", "_")
                if prefix[0].isalpha() or prefix[0] == '_':
                    hash_value = prefix+hash_value
                else:
                    hash_value = "Prefix must start with 'alphabat' or '_'"
            else:
                hash_value = "layout_" + hash_value
        except Exception as e:
            hash_value = e
        return hash_value



from __future__ import absolute_import
import os.path
from os.path import relpath
import shutil
import os
import sys
import jinja2

from blendedUxLang import BlendedEnvironment
from blendedUx.blended_flask.blended_hostlib.exceptions import BlendedException
from blendedUx.blended_flask.utils import *

env = BlendedEnvironment()

if not os.path.exists(IMAGE_CACHE_DIR):
    os.makedirs(IMAGE_CACHE_DIR)

def get_template(file_name):
    """
    """
    file_extension = file_name.split('.')[-1].lower()
    path_1 = "./templates/" + file_name
    try:
        with open(path_1, 'r', encoding='utf-8') as content_file:
                file_data = content_file.read()
                return file_data
    except Exception as e:
        if file_extension:
            print(e)
            return ''
        print(file_name)

def render_code(file_content, context):
    """
    :param file_content: compiled template content
    :param context: theme_object or context
    :return: rendered content of template.
    """
    template = env.from_string(file_content)
    return template.render(context)

def image(image_dict_obj, height=None, width=None, filters=None):
    """
    """
    if filters:
        if ((height == 0) or height) and ((width == 0) or width):
            pass
        elif not (height and width):
            print("Please pass height and width before using filters! error in:%s)" % image_dict_obj)
            sys.exit(0)
    elif height or width:
        filters = 'series(fill, crop(smart))'
    else:
        pass
    try:
        if height:
            float(height)
        if width:
            float(width)
    except ValueError:
        raise BlendedException("Please pass numerical value of height and width before using filters. "
                               "Default values are (0, 0)! error in:%s)" % image_dict_obj)
    try:
        relative_path = image_dict_obj.get('path')
        output = get_image_path(relative_path, height=height, width=width, filters=filters)
        url = "blended_static/media/%s" % (relpath(output, IMAGE_CACHE_DIR))
    except jinja2.exceptions.UndefinedError as exc:
        filename = exc.message.rsplit('attribute ')[1].replace("'", "")
        url = "blended_static/media/not_found/%s" % (filename)
    return url

def css(theme_object):
    """
    function to generate css file path dynamically.
    """
    css_path = theme_object.get('meta', {}).get('css', {})
    source_path = css_path.get('compile_targets')
    rendered_require = css_path.get('render_required')
    context = {'theme': dict(theme_object)}
    try:
        css = theme_object['css']['blendedcss']
        file_path = os.path.join(COMPILE_DIR, 'blendedcss')
        if(rendered_require):
            css = render_code(css, context)
        try:
            path = save_css(file_path, css)
        except BlendedException as exc:
            raise BlendedException(exc)
        else:
            split_path = path.split(COMPILE_DIR)
            css_file_url = split_path[1]
            css_file_url = "/blended_static/css/blended/%s" % (css_file_url)
            return css_file_url
    except Exception as e:
        raise(e)

def url_for(path, filename=""):
    """
    """
    return "/%s/%s" % (path, filename)

def blended_home():
    """
    """
    return 'http://127.0.0.1:5000/'

def nav_links(theme, filename):
    return "%s" % (filename)

env.globals.update({'css': css,
                    'css_links': css,
                    'nav_links': nav_links,
                    'image': image,
                    "url_for": url_for,
                    'home': blended_home
                    })



from blendedUx.blended_flask.blended_hostlib.backend import FileSystemBackend
from blendedUx.blended_flask.blended_hostlib.network import Network
from blendedUx.blended_flask.blended_hostlib.controller import Controller
from flask import Flask, send_from_directory, send_file
from blendedUx.blended_flask.settings import blended_static_dir
import os

class BlendedFlask():
    """
    """
    def __init__(self,app, package_dir):
        """
        """
        self.package_dir = package_dir
        @app.route("/blended_static/<path:path>")
        def blended_static(path):
            try:
                return send_from_directory(blended_static_dir, path)
            except Exception:
                return send_file(os.path.join(blended_static_dir, path))

    def load(self, theme_name, user_slug):
        """
        """
        network = Network()
        backend = FileSystemBackend()
        controller = Controller(network, backend)
        backend.blended_dir = self.package_dir
        context = controller.context_loader(theme_name, user_slug)
        return context

    def login(self, user_slug, password):
        """
        """
        pass



from __future__ import absolute_import
import re
import math

from PIL import Image, ImageFilter, ImageOps, ImageFile

CUT_SIZE = 10

class TransformImage(object):
    """
    """
    FILTER_MAP = {
      "monochrome": lambda x, y: x._monochrome(*y),
      "alphachrome": lambda x, y: x._alphachrome(*y),
      "nocrop": lambda x, y: x._nocrop(*y),
      "warp": lambda x, y: x._warp(*y),
      "fit": lambda x, y: x._fit(*y),
      "fill": lambda x, y: x._fill(*y),
      "resize": lambda x, y: x._resize(*y),
      "crop": lambda x, y: x._crop(*y),
      "series": lambda x, y: x._do_series(*y),
    }

    def __init__(self, fil,  height=None, width=None):
        self.image = Image.open(fil)
        self.image.load()
        self.format = self.image.format
        if self.format in ('GIF', 'gif') and 'transparency' in self.image.info:
            self.transparency = self.image.info['transparency']
        else:
            self.transparency = None
        self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0
        if(width):
            self.width = width*1.0
        else:
            self.width = self.w
        if(height):
            self.height = height*1.0
        else:
            self.height = self.h

        if self.width == 0:
            self.width = self.w
        if self.height == 0:
            self.height = self.h

    def save_file(self, fileloc, format=None, **kwargs):
        if format is None:
            format = self.format
        if self.transparency is not None:

            if 'transparency' in kwargs:
                transparency = kwargs['transparency']
                kwargs = kwargs.copy()
                del kwargs['transparency']
            else:
                transparency = self.transparency
            self.image.save(fileloc, format, transparency=transparency, optimize=True, **kwargs)
        else:
            self.image.save(fileloc, format, optimize=True, **kwargs)

    def apply_filter(self, filter):
        m = re.match('(\w+)(\((.*)\))?$', filter)
        filter_name = m.group(1)
        args = m.group(3)
        if args is not None:
            args = [item.strip() for item in args.split(',')]
        else:
            args = []
        clean_args = []
        # convert integer arguments into integer values
        for i in range(len(args)):
            try:
                val = int(args[i])
                clean_args.append(val)
            except:
                if args[i]:
                    clean_args.append(args[i])
        filter_func = TransformImage.FILTER_MAP.get(filter_name, None)
        if filter_func is not None:
            filter_func(self, clean_args)
        return self

    def _do_series(self, *filters):
        for filter in filters:
            self.apply_filter(filter)

    def __getrgba(self, value):
        match = re.search(r'^#?[0-9a-fA-F]{3,3}([0-9a-fA-F]{3,3})?([0-9a-fA-F]{2,2})?$', value)
        if match is None:
            raise Exception("Value '%s' is not a valid rgb color hexidecimal." % value)
        if value[0] == '#':
            value = value[1:]

        a = 255
        if len(value) == 3:
            colors = value[0]*2, value[1]*2, value[2]*2
            r, g, b = map(lambda x: int(x, 16), colors)
        elif len(value) == 6:
            colors = value[:2], value[2:4], value[4:6]
            r, g, b = map(lambda x: int(x, 16), colors)
        elif len(value) == 8:
            colors = value[:2], value[2:4], value[4:6], value[6:8]
            r, g, b, a = map(lambda x: int(x, 16), colors)
        else:
            raise Exception("Value '%s' is not a valid rgb color hexidecimal." % s)

        for x in r, g, b, a:
            if x < 0 or x > 255:
                raise TypeError("Somehow a hex value is to greater than 255 or negative.")

        return (r, g, b, a)

    def __make_linear_ramp(self, white):
        ramp = []
        r, g, b = white
        for i in range(255):
            ramp.extend((int(r*i/255), int(g*i/255), int(b*i/255)))
        return ramp

    def _monochrome(self, chrome="FFF0C0", *args):
        r, g, b, a = self.__getrgba(chrome)
        palette = self.__make_linear_ramp((r, g, b))
        # Get the alpha layer:
        if self.image.mode != "RGBA":
            self.image = self.image.convert("RGBA")
        alpha = self.image.split()[-1]
        # Make black and white for applying monochrome.
        self.image = self.image.convert("L")
        # self.image = self.image.convert("1")
        # self.image.save("./image_output/"+output_path+"new", "BMP", quality=100, optimize=True, progressive=True)
        # apply contrast enhancement here
        self.image = ImageOps.autocontrast(self.image)
        # apply monochrome palette
        self.image.putpalette(palette)
        # convert back to RGB
        self.image = self.image.convert("RGBA")
        # put the alpha back onto the image
        self.image.putalpha(alpha)

    def _nocrop(self):
        '''
        :return:
        '''
        raise NotImplementedError

    def _alphachrome(self, chrome="3377FF", *args):
        r, g, b, a = self.__getrgba(chrome)

        # Get the alpha layer:
        if self.image.mode != "RGBA":
            self.image = self.image.convert("RGBA")
        alpha = self.image.split()[-1]

        # generate a solid field of color for the new image.
        colorfield = Image.new('RGBA', (int(self.width), int(self.height)), (r, g, b, a))

        # save the solid color as the image, then apply the alpha to it.
        self.image = colorfield
        self.image.putalpha(alpha)

    def _warp(self, *args):
        ratio_w = (1.0*self.width)/(self.w)
        ratio_h = (1.0*self.height)/(self.h)

        self.image = self.image.resize((int(self.w*ratio_w), int(self.h*ratio_h)), Image.ANTIALIAS)
        self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0

    def _fit(self):
        '''
        _fit shinks the image down so that it fits entirely inside the bounding box.

        The size of the image will be proportional to its original size and
        smaller than the bounding box, or possibly the same size.
        '''

        if (self.w/self.width > self.h/self.height):
            ratio = self.width/self.w
        else:
            ratio = self.height/self.h

        self.image = self.image.resize((int(self.w*ratio), int(self.h*ratio)), Image.ANTIALIAS)
        self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0

    def _fill(self, *args):
        if (self.w/self.width < self.h/self.height):
            ratio = self.width/self.w
        else:
            ratio = self.height/self.h

        self.image = self.image.resize((int(self.w*ratio), int(self.h*ratio)), Image.ANTIALIAS)
        self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0

    def _resize(self, percent=100):
        '''
        _resize shinks the image down to a percentage of its size difference with the
        bounding box, as specified by the "percent" paramater.  The default value for
        "percent" is 100
        '''
        if (self.w/self.width < self.h/self.height):
            diff = abs(self.width-self.w)*percent/100.0
            ratio = (self.width+diff)/self.w
        else:
            diff = abs(self.height-self.h)*percent/100.0
            ratio = (1.0*self.height+diff)/(self.h*1.0)

        self.image = self.image.resize((int(self.w*ratio), int(self.h*ratio), ))
        self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0

    def __entropy(self, image=None):
        image = image if image is not None else self.image
        hist = image.histogram()
        hist_size = sum(hist)

        if hist_size == 0:
            entropy = 0
        else:
            hist = [float(h) / hist_size for h in hist]
            entropy = -sum([p * math.log(p, 2) for p in hist if p != 0])

        return entropy

    def __smartcrop(self):
        self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0
        wdiff = self.w - self.width
        hdiff = self.h - self.height

        while wdiff > 0 or hdiff > 0:
            if wdiff >= hdiff:
                slice_width = int(min(wdiff, CUT_SIZE))
                left = self.image.crop((0, 0, slice_width, int(self.h)))
                right = self.image.crop((int(self.w-slice_width), 0, int(self.w), int(self.h)))

                if self.__entropy(left) > self.__entropy(right):
                    cropbox = (0, 0, int(self.w - slice_width), int(self.h))
                else:
                    cropbox = (slice_width, 0, int(self.w), int(self.h))

            else:
                slice_height = int(min(hdiff, CUT_SIZE))
                top = self.image.crop((0, 0, int(self.w), slice_height))
                bottom = self.image.crop((0, int(self.h - slice_height), int(self.w), int(self.h)))

                if self.__entropy(top) > self.__entropy(bottom):
                    cropbox = (0, 0, int(self.w), int(self.h - slice_height))
                else:
                    cropbox = (0, slice_height, int(self.w), int(self.h))

            self.image = self.image.crop(cropbox)
            self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0

            wdiff = self.w - self.width
            hdiff = self.h - self.height

    def _crop(self, retain="center", background=None):
        '''
        If a "background" is specified (in the string-hex format RRGGBBAA), then the image
        will be sized to match the bounding box, with extra space filled by the specified
        "background" color+alpha, and with the image placed at the location specified by
        the "retain" parameter.

        Otherwise, the crop will simply cut away aspects of the image that are outside
        the bounding box,
        '''
        if retain == "center":
            crop_w = (self.w-self.width)/2
            crop_h = (self.h-self.height)/2

            cropbox = (crop_w, crop_h, crop_w+self.width, crop_h+self.height)
            cropbox = map(lambda x: int(x), cropbox)

            self.image = self.image.crop(cropbox)
            self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0

        elif retain == "top-left":
            cropbox = (0, 0, int(self.width), int(self.height))

            self.image = self.image.crop(cropbox)
            self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0

        elif retain == "bottom-right":
            cropbox = (self.w - self.width, self.h - self.height, self.w, self.h)
            cropbox = map(lambda x: int(x), cropbox)

            self.image = self.image.crop(cropbox)
            self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0

        elif retain == "smart":
            self.__smartcrop()

        else:
            raise TypeError("Crop instruction retain=%s is not understood." % retain)

        if background is not None and background != '':
            self._background(retain, background)

    def _background(self, placement="center", background="FFFFFF00"):

        r, g, b, a = self.__getrgba(background)
        background = Image.new('RGBA', (int(self.width), int(self.height)), (r, g, b, a))

        if placement == "center":
            box = (self.width-self.w)/2, (self.height-self.h)/2
        if placement == "top-left":
            box = (0, 0)
        if placement == "bottom-right":
            box = (self.width-self.w, self.height-self.h)
        box = tuple(map(lambda x: int(x), box))

        background.paste(self.image, box)
        self.image = background
        self.w, self.h = self.image.size[0]*1.0, self.image.size[1]*1.0




# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class InitialPackage(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, href=None, name=None, account=None, slug=None, acquisition_type=None, is_published=None, canonical=None, is_active=None, version=None, title=None, user=None):
        """
        InitialPackage - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'href': 'str',
            'name': 'str',
            'account': 'str',
            'slug': 'str',
            'acquisition_type': 'str',
            'is_published': 'bool',
            'canonical': 'object',
            'is_active': 'bool',
            'version': 'str',
            'title': 'str',
            'user': 'str'
        }

        self.attribute_map = {
            'href': 'href',
            'name': 'name',
            'account': 'account',
            'slug': 'slug',
            'acquisition_type': 'acquisition_type',
            'is_published': 'is_published',
            'canonical': 'canonical',
            'is_active': 'is_active',
            'version': 'version',
            'title': 'title',
            'user': 'user'
        }

        self._href = href
        self._name = name
        self._account = account
        self._slug = slug
        self._acquisition_type = acquisition_type
        self._is_published = is_published
        self._canonical = canonical
        self._is_active = is_active
        self._version = version
        self._title = title
        self._user = user

    @property
    def href(self):
        """
        Gets the href of this InitialPackage.
        current object reference url

        :return: The href of this InitialPackage.
        :rtype: str
        """
        return self._href

    @href.setter
    def href(self, href):
        """
        Sets the href of this InitialPackage.
        current object reference url

        :param href: The href of this InitialPackage.
        :type: str
        """

        self._href = href

    @property
    def name(self):
        """
        Gets the name of this InitialPackage.
        package name.

        :return: The name of this InitialPackage.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this InitialPackage.
        package name.

        :param name: The name of this InitialPackage.
        :type: str
        """

        self._name = name

    @property
    def account(self):
        """
        Gets the account of this InitialPackage.
        package creator name.

        :return: The account of this InitialPackage.
        :rtype: str
        """
        return self._account

    @account.setter
    def account(self, account):
        """
        Sets the account of this InitialPackage.
        package creator name.

        :param account: The account of this InitialPackage.
        :type: str
        """

        self._account = account

    @property
    def slug(self):
        """
        Gets the slug of this InitialPackage.
        value used to uniquely identify package.

        :return: The slug of this InitialPackage.
        :rtype: str
        """
        return self._slug

    @slug.setter
    def slug(self, slug):
        """
        Sets the slug of this InitialPackage.
        value used to uniquely identify package.

        :param slug: The slug of this InitialPackage.
        :type: str
        """

        self._slug = slug

    @property
    def acquisition_type(self):
        """
        Gets the acquisition_type of this InitialPackage.
        acquisition type

        :return: The acquisition_type of this InitialPackage.
        :rtype: str
        """
        return self._acquisition_type

    @acquisition_type.setter
    def acquisition_type(self, acquisition_type):
        """
        Sets the acquisition_type of this InitialPackage.
        acquisition type

        :param acquisition_type: The acquisition_type of this InitialPackage.
        :type: str
        """

        self._acquisition_type = acquisition_type

    @property
    def is_published(self):
        """
        Gets the is_published of this InitialPackage.
        package is published or not.

        :return: The is_published of this InitialPackage.
        :rtype: bool
        """
        return self._is_published

    @is_published.setter
    def is_published(self, is_published):
        """
        Sets the is_published of this InitialPackage.
        package is published or not.

        :param is_published: The is_published of this InitialPackage.
        :type: bool
        """

        self._is_published = is_published

    @property
    def canonical(self):
        """
        Gets the canonical of this InitialPackage.
        Canonical Package Details.

        :return: The canonical of this InitialPackage.
        :rtype: object
        """
        return self._canonical

    @canonical.setter
    def canonical(self, canonical):
        """
        Sets the canonical of this InitialPackage.
        Canonical Package Details.

        :param canonical: The canonical of this InitialPackage.
        :type: object
        """

        self._canonical = canonical

    @property
    def is_active(self):
        """
        Gets the is_active of this InitialPackage.
        Suggested the client of Active Theme.

        :return: The is_active of this InitialPackage.
        :rtype: bool
        """
        return self._is_active

    @is_active.setter
    def is_active(self, is_active):
        """
        Sets the is_active of this InitialPackage.
        Suggested the client of Active Theme.

        :param is_active: The is_active of this InitialPackage.
        :type: bool
        """

        self._is_active = is_active

    @property
    def version(self):
        """
        Gets the version of this InitialPackage.
        label detail

        :return: The version of this InitialPackage.
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """
        Sets the version of this InitialPackage.
        label detail

        :param version: The version of this InitialPackage.
        :type: str
        """

        self._version = version

    @property
    def title(self):
        """
        Gets the title of this InitialPackage.
        package title

        :return: The title of this InitialPackage.
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """
        Sets the title of this InitialPackage.
        package title

        :param title: The title of this InitialPackage.
        :type: str
        """

        self._title = title

    @property
    def user(self):
        """
        Gets the user of this InitialPackage.
        package.user.user_name

        :return: The user of this InitialPackage.
        :rtype: str
        """
        return self._user

    @user.setter
    def user(self, user):
        """
        Sets the user of this InitialPackage.
        package.user.user_name

        :param user: The user of this InitialPackage.
        :type: str
        """

        self._user = user

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from __future__ import absolute_import

# import models into model package
from .account import Account
from .account_invite_pending import AccountInvitePending
from .account_invites_pending_list import AccountInvitesPendingList
from .accounts import Accounts
from .acquisition import Acquisition
from .acquisition_response import AcquisitionResponse
from .acquisition_response_data import AcquisitionResponseData
from .activationchallenge import Activationchallenge
from .active_account import ActiveAccount
from .add_account import AddAccount
from .add_admin import AddAdmin
from .add_license import AddLicense
from .add_pending import AddPending
from .add_rating import AddRating
from .all_license_price import AllLicensePrice
from .blended_package import BlendedPackage
from .body import Body
from .body_1 import Body1
from .bundled_packages import BundledPackages
from .bundled_packages_error import BundledPackagesError
from .canonical_version import CanonicalVersion
from .change_current_account import ChangeCurrentAccount
from .change_privilege import ChangePrivilege
from .create_account import CreateAccount
from .create_canonical import CreateCanonical
from .create_package import CreatePackage
from .create_version import CreateVersion
from .dependencies import Dependencies
from .dependencies_list import DependenciesList
from .dependency_package_detail import DependencyPackageDetail
from .dependency_package_detail_dependency_of_package import DependencyPackageDetailDependencyOfPackage
from .documents import Documents
from .error import Error
from .error_response400 import ErrorResponse400
from .error_response401 import ErrorResponse401
from .error_response409 import ErrorResponse409
from .error_response412 import ErrorResponse412
from .error_response500 import ErrorResponse500
from .failed_dependency_error import FailedDependencyError
from .hyper_link_account import HyperLinkAccount
from .hyper_link_canonical_version import HyperLinkCanonicalVersion
from .hyper_link_current_account import HyperLinkCurrentAccount
from .hyper_link_dependency_packages import HyperLinkDependencyPackages
from .hyper_link_draft import HyperLinkDraft
from .hyper_link_licenses import HyperLinkLicenses
from .hyper_link_pending_user import HyperLinkPendingUser
from .hyper_link_ratings import HyperLinkRatings
from .hyper_link_user import HyperLinkUser
from .hyper_link_validators import HyperLinkValidators
from .initial_package import InitialPackage
from .initial_packages import InitialPackages
from .inline_response_200 import InlineResponse200
from .license import License
from .like import Like
from .login import Login
from .logout import Logout
from .new_draft import NewDraft
from .operands import Operands
from .package_detail_data import PackageDetailData
from .package_images import PackageImages
from .packages import Packages
from .pending_user import PendingUser
from .pending_users import PendingUsers
from .publication import Publication
from .publication_error import PublicationError
from .publication_validation_error import PublicationValidationError
from .publication_version import PublicationVersion
from .publication_version_error import PublicationVersionError
from .publication_versions import PublicationVersions
from .publication_versions_error import PublicationVersionsError
from .publications import Publications
from .publications_error import PublicationsError
from .rating import Rating
from .reject_acquisition_package import RejectAcquisitionPackage
from .remove_pending import RemovePending
from .resend_account_verification_link import ResendAccountVerificationLink
from .resend_invite import ResendInvite
from .reset_password import ResetPassword
from .search_result import SearchResult
from .search_results import SearchResults
from .session_data import SessionData
from .share_approval_list_error import ShareApprovalListError
from .shared_package_account import SharedPackageAccount
from .shared_package_accounts_list import SharedPackageAccountsList
from .update_account import UpdateAccount
from .update_draft import UpdateDraft
from .update_license import UpdateLicense
from .update_package import UpdatePackage
from .update_publication import UpdatePublication
from .update_version import UpdateVersion
from .upload_media import UploadMedia
from .user import User
from .users import Users
from .validator import Validator
from .validator_detail import ValidatorDetail
from .validator_plan import ValidatorPlan
from .validator_sets_in_badge import ValidatorSetsInBadge
from .version import Version
from .versions import Versions



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class BundledPackages(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, label=None, package=None, license_name=None, license_type=None, price=None, bundles_price=None):
        """
        BundledPackages - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'label': 'str',
            'package': 'str',
            'license_name': 'str',
            'license_type': 'str',
            'price': 'float',
            'bundles_price': 'float'
        }

        self.attribute_map = {
            'label': 'label',
            'package': 'package',
            'license_name': 'license_name',
            'license_type': 'license_type',
            'price': 'price',
            'bundles_price': 'bundles_price'
        }

        self._label = label
        self._package = package
        self._license_name = license_name
        self._license_type = license_type
        self._price = price
        self._bundles_price = bundles_price

    @property
    def label(self):
        """
        Gets the label of this BundledPackages.
        label of package version

        :return: The label of this BundledPackages.
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """
        Sets the label of this BundledPackages.
        label of package version

        :param label: The label of this BundledPackages.
        :type: str
        """

        self._label = label

    @property
    def package(self):
        """
        Gets the package of this BundledPackages.
        fully qualified name of package

        :return: The package of this BundledPackages.
        :rtype: str
        """
        return self._package

    @package.setter
    def package(self, package):
        """
        Sets the package of this BundledPackages.
        fully qualified name of package

        :param package: The package of this BundledPackages.
        :type: str
        """

        self._package = package

    @property
    def license_name(self):
        """
        Gets the license_name of this BundledPackages.
        license name

        :return: The license_name of this BundledPackages.
        :rtype: str
        """
        return self._license_name

    @license_name.setter
    def license_name(self, license_name):
        """
        Sets the license_name of this BundledPackages.
        license name

        :param license_name: The license_name of this BundledPackages.
        :type: str
        """

        self._license_name = license_name

    @property
    def license_type(self):
        """
        Gets the license_type of this BundledPackages.
        license type

        :return: The license_type of this BundledPackages.
        :rtype: str
        """
        return self._license_type

    @license_type.setter
    def license_type(self, license_type):
        """
        Sets the license_type of this BundledPackages.
        license type

        :param license_type: The license_type of this BundledPackages.
        :type: str
        """

        self._license_type = license_type

    @property
    def price(self):
        """
        Gets the price of this BundledPackages.
        license type

        :return: The price of this BundledPackages.
        :rtype: float
        """
        return self._price

    @price.setter
    def price(self, price):
        """
        Sets the price of this BundledPackages.
        license type

        :param price: The price of this BundledPackages.
        :type: float
        """

        self._price = price

    @property
    def bundles_price(self):
        """
        Gets the bundles_price of this BundledPackages.
        total price of bundled packages

        :return: The bundles_price of this BundledPackages.
        :rtype: float
        """
        return self._bundles_price

    @bundles_price.setter
    def bundles_price(self, bundles_price):
        """
        Sets the bundles_price of this BundledPackages.
        total price of bundled packages

        :param bundles_price: The bundles_price of this BundledPackages.
        :type: float
        """

        self._bundles_price = bundles_price

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class ErrorResponse500(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, status_code=None, message=None, package_detail_data=None, redirect_url=None):
        """
        ErrorResponse500 - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'status_code': 'int',
            'message': 'str',
            'package_detail_data': 'object',
            'redirect_url': 'str'
        }

        self.attribute_map = {
            'status_code': 'status_code',
            'message': 'message',
            'package_detail_data': 'packageDetailData',
            'redirect_url': 'redirect_url'
        }

        self._status_code = status_code
        self._message = message
        self._package_detail_data = package_detail_data
        self._redirect_url = redirect_url

    @property
    def status_code(self):
        """
        Gets the status_code of this ErrorResponse500.
        Error Custom status code.

        :return: The status_code of this ErrorResponse500.
        :rtype: int
        """
        return self._status_code

    @status_code.setter
    def status_code(self, status_code):
        """
        Sets the status_code of this ErrorResponse500.
        Error Custom status code.

        :param status_code: The status_code of this ErrorResponse500.
        :type: int
        """

        self._status_code = status_code

    @property
    def message(self):
        """
        Gets the message of this ErrorResponse500.
        error message.

        :return: The message of this ErrorResponse500.
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """
        Sets the message of this ErrorResponse500.
        error message.

        :param message: The message of this ErrorResponse500.
        :type: str
        """

        self._message = message

    @property
    def package_detail_data(self):
        """
        Gets the package_detail_data of this ErrorResponse500.
        details of the package.

        :return: The package_detail_data of this ErrorResponse500.
        :rtype: object
        """
        return self._package_detail_data

    @package_detail_data.setter
    def package_detail_data(self, package_detail_data):
        """
        Sets the package_detail_data of this ErrorResponse500.
        details of the package.

        :param package_detail_data: The package_detail_data of this ErrorResponse500.
        :type: object
        """

        self._package_detail_data = package_detail_data

    @property
    def redirect_url(self):
        """
        Gets the redirect_url of this ErrorResponse500.
        url if redirection to website is needed at all.

        :return: The redirect_url of this ErrorResponse500.
        :rtype: str
        """
        return self._redirect_url

    @redirect_url.setter
    def redirect_url(self, redirect_url):
        """
        Sets the redirect_url of this ErrorResponse500.
        url if redirection to website is needed at all.

        :param redirect_url: The redirect_url of this ErrorResponse500.
        :type: str
        """

        self._redirect_url = redirect_url

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class DependencyPackageDetail(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, package=None, label=None, title=None, third_party=None, get_or_purchased=None, thumbnail_image=None, thumbnail_image_hash=None, license_price=None, type=None, dependency_of_package=None):
        """
        DependencyPackageDetail - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'package': 'str',
            'label': 'str',
            'title': 'str',
            'third_party': 'bool',
            'get_or_purchased': 'bool',
            'thumbnail_image': 'str',
            'thumbnail_image_hash': 'str',
            'license_price': 'int',
            'type': 'str',
            'dependency_of_package': 'DependencyPackageDetailDependencyOfPackage'
        }

        self.attribute_map = {
            'package': 'package',
            'label': 'label',
            'title': 'title',
            'third_party': 'third_party',
            'get_or_purchased': 'get_or_purchased',
            'thumbnail_image': 'thumbnail_image',
            'thumbnail_image_hash': 'thumbnail_image_hash',
            'license_price': 'license_price',
            'type': 'type',
            'dependency_of_package': 'dependency_of_package'
        }

        self._package = package
        self._label = label
        self._title = title
        self._third_party = third_party
        self._get_or_purchased = get_or_purchased
        self._thumbnail_image = thumbnail_image
        self._thumbnail_image_hash = thumbnail_image_hash
        self._license_price = license_price
        self._type = type
        self._dependency_of_package = dependency_of_package

    @property
    def package(self):
        """
        Gets the package of this DependencyPackageDetail.
        fully qualified package name

        :return: The package of this DependencyPackageDetail.
        :rtype: str
        """
        return self._package

    @package.setter
    def package(self, package):
        """
        Sets the package of this DependencyPackageDetail.
        fully qualified package name

        :param package: The package of this DependencyPackageDetail.
        :type: str
        """

        self._package = package

    @property
    def label(self):
        """
        Gets the label of this DependencyPackageDetail.
        version label of dependency package

        :return: The label of this DependencyPackageDetail.
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """
        Sets the label of this DependencyPackageDetail.
        version label of dependency package

        :param label: The label of this DependencyPackageDetail.
        :type: str
        """

        self._label = label

    @property
    def title(self):
        """
        Gets the title of this DependencyPackageDetail.
        title of the dependency package

        :return: The title of this DependencyPackageDetail.
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """
        Sets the title of this DependencyPackageDetail.
        title of the dependency package

        :param title: The title of this DependencyPackageDetail.
        :type: str
        """

        self._title = title

    @property
    def third_party(self):
        """
        Gets the third_party of this DependencyPackageDetail.
        boolean field to identify third party package.

        :return: The third_party of this DependencyPackageDetail.
        :rtype: bool
        """
        return self._third_party

    @third_party.setter
    def third_party(self, third_party):
        """
        Sets the third_party of this DependencyPackageDetail.
        boolean field to identify third party package.

        :param third_party: The third_party of this DependencyPackageDetail.
        :type: bool
        """

        self._third_party = third_party

    @property
    def get_or_purchased(self):
        """
        Gets the get_or_purchased of this DependencyPackageDetail.
        boolean field to identify purchased packages.

        :return: The get_or_purchased of this DependencyPackageDetail.
        :rtype: bool
        """
        return self._get_or_purchased

    @get_or_purchased.setter
    def get_or_purchased(self, get_or_purchased):
        """
        Sets the get_or_purchased of this DependencyPackageDetail.
        boolean field to identify purchased packages.

        :param get_or_purchased: The get_or_purchased of this DependencyPackageDetail.
        :type: bool
        """

        self._get_or_purchased = get_or_purchased

    @property
    def thumbnail_image(self):
        """
        Gets the thumbnail_image of this DependencyPackageDetail.
        thumbnail_image of the dependency package

        :return: The thumbnail_image of this DependencyPackageDetail.
        :rtype: str
        """
        return self._thumbnail_image

    @thumbnail_image.setter
    def thumbnail_image(self, thumbnail_image):
        """
        Sets the thumbnail_image of this DependencyPackageDetail.
        thumbnail_image of the dependency package

        :param thumbnail_image: The thumbnail_image of this DependencyPackageDetail.
        :type: str
        """

        self._thumbnail_image = thumbnail_image

    @property
    def thumbnail_image_hash(self):
        """
        Gets the thumbnail_image_hash of this DependencyPackageDetail.
        thumbnail_image_hash for base64 lookup in thumbnail_image_data

        :return: The thumbnail_image_hash of this DependencyPackageDetail.
        :rtype: str
        """
        return self._thumbnail_image_hash

    @thumbnail_image_hash.setter
    def thumbnail_image_hash(self, thumbnail_image_hash):
        """
        Sets the thumbnail_image_hash of this DependencyPackageDetail.
        thumbnail_image_hash for base64 lookup in thumbnail_image_data

        :param thumbnail_image_hash: The thumbnail_image_hash of this DependencyPackageDetail.
        :type: str
        """

        self._thumbnail_image_hash = thumbnail_image_hash

    @property
    def license_price(self):
        """
        Gets the license_price of this DependencyPackageDetail.
        price of the license which the dependency package is part of

        :return: The license_price of this DependencyPackageDetail.
        :rtype: int
        """
        return self._license_price

    @license_price.setter
    def license_price(self, license_price):
        """
        Sets the license_price of this DependencyPackageDetail.
        price of the license which the dependency package is part of

        :param license_price: The license_price of this DependencyPackageDetail.
        :type: int
        """

        self._license_price = license_price

    @property
    def type(self):
        """
        Gets the type of this DependencyPackageDetail.
        package type of dependency package

        :return: The type of this DependencyPackageDetail.
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Sets the type of this DependencyPackageDetail.
        package type of dependency package

        :param type: The type of this DependencyPackageDetail.
        :type: str
        """

        self._type = type

    @property
    def dependency_of_package(self):
        """
        Gets the dependency_of_package of this DependencyPackageDetail.


        :return: The dependency_of_package of this DependencyPackageDetail.
        :rtype: DependencyPackageDetailDependencyOfPackage
        """
        return self._dependency_of_package

    @dependency_of_package.setter
    def dependency_of_package(self, dependency_of_package):
        """
        Sets the dependency_of_package of this DependencyPackageDetail.


        :param dependency_of_package: The dependency_of_package of this DependencyPackageDetail.
        :type: DependencyPackageDetailDependencyOfPackage
        """

        self._dependency_of_package = dependency_of_package

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class HyperLinkDraft(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, href=None, pk=None, package=None, draft_name=None, modified_date=None, created_date=None, package_hash=None, user=None, validatorplan=None, documents=None):
        """
        HyperLinkDraft - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'href': 'str',
            'pk': 'int',
            'package': 'int',
            'draft_name': 'str',
            'modified_date': 'datetime',
            'created_date': 'datetime',
            'package_hash': 'str',
            'user': 'str',
            'validatorplan': 'object',
            'documents': 'object'
        }

        self.attribute_map = {
            'href': 'href',
            'pk': 'pk',
            'package': 'package',
            'draft_name': 'draft_name',
            'modified_date': 'modified_date',
            'created_date': 'created_date',
            'package_hash': 'package_hash',
            'user': 'user',
            'validatorplan': 'validatorplan',
            'documents': 'documents'
        }

        self._href = href
        self._pk = pk
        self._package = package
        self._draft_name = draft_name
        self._modified_date = modified_date
        self._created_date = created_date
        self._package_hash = package_hash
        self._user = user
        self._validatorplan = validatorplan
        self._documents = documents

    @property
    def href(self):
        """
        Gets the href of this HyperLinkDraft.
        current object reference url

        :return: The href of this HyperLinkDraft.
        :rtype: str
        """
        return self._href

    @href.setter
    def href(self, href):
        """
        Sets the href of this HyperLinkDraft.
        current object reference url

        :param href: The href of this HyperLinkDraft.
        :type: str
        """

        self._href = href

    @property
    def pk(self):
        """
        Gets the pk of this HyperLinkDraft.
        unique identifier of draft.

        :return: The pk of this HyperLinkDraft.
        :rtype: int
        """
        return self._pk

    @pk.setter
    def pk(self, pk):
        """
        Sets the pk of this HyperLinkDraft.
        unique identifier of draft.

        :param pk: The pk of this HyperLinkDraft.
        :type: int
        """

        self._pk = pk

    @property
    def package(self):
        """
        Gets the package of this HyperLinkDraft.
        package pk.

        :return: The package of this HyperLinkDraft.
        :rtype: int
        """
        return self._package

    @package.setter
    def package(self, package):
        """
        Sets the package of this HyperLinkDraft.
        package pk.

        :param package: The package of this HyperLinkDraft.
        :type: int
        """

        self._package = package

    @property
    def draft_name(self):
        """
        Gets the draft_name of this HyperLinkDraft.
        draft name.

        :return: The draft_name of this HyperLinkDraft.
        :rtype: str
        """
        return self._draft_name

    @draft_name.setter
    def draft_name(self, draft_name):
        """
        Sets the draft_name of this HyperLinkDraft.
        draft name.

        :param draft_name: The draft_name of this HyperLinkDraft.
        :type: str
        """

        self._draft_name = draft_name

    @property
    def modified_date(self):
        """
        Gets the modified_date of this HyperLinkDraft.
        last modified date.

        :return: The modified_date of this HyperLinkDraft.
        :rtype: datetime
        """
        return self._modified_date

    @modified_date.setter
    def modified_date(self, modified_date):
        """
        Sets the modified_date of this HyperLinkDraft.
        last modified date.

        :param modified_date: The modified_date of this HyperLinkDraft.
        :type: datetime
        """

        self._modified_date = modified_date

    @property
    def created_date(self):
        """
        Gets the created_date of this HyperLinkDraft.
        package draft creation date.

        :return: The created_date of this HyperLinkDraft.
        :rtype: datetime
        """
        return self._created_date

    @created_date.setter
    def created_date(self, created_date):
        """
        Sets the created_date of this HyperLinkDraft.
        package draft creation date.

        :param created_date: The created_date of this HyperLinkDraft.
        :type: datetime
        """

        self._created_date = created_date

    @property
    def package_hash(self):
        """
        Gets the package_hash of this HyperLinkDraft.
        package hash

        :return: The package_hash of this HyperLinkDraft.
        :rtype: str
        """
        return self._package_hash

    @package_hash.setter
    def package_hash(self, package_hash):
        """
        Sets the package_hash of this HyperLinkDraft.
        package hash

        :param package_hash: The package_hash of this HyperLinkDraft.
        :type: str
        """

        self._package_hash = package_hash

    @property
    def user(self):
        """
        Gets the user of this HyperLinkDraft.
        package draft creator name.

        :return: The user of this HyperLinkDraft.
        :rtype: str
        """
        return self._user

    @user.setter
    def user(self, user):
        """
        Sets the user of this HyperLinkDraft.
        package draft creator name.

        :param user: The user of this HyperLinkDraft.
        :type: str
        """

        self._user = user

    @property
    def validatorplan(self):
        """
        Gets the validatorplan of this HyperLinkDraft.
        Detail view of rating.

        :return: The validatorplan of this HyperLinkDraft.
        :rtype: object
        """
        return self._validatorplan

    @validatorplan.setter
    def validatorplan(self, validatorplan):
        """
        Sets the validatorplan of this HyperLinkDraft.
        Detail view of rating.

        :param validatorplan: The validatorplan of this HyperLinkDraft.
        :type: object
        """

        self._validatorplan = validatorplan

    @property
    def documents(self):
        """
        Gets the documents of this HyperLinkDraft.
        Detail view of rating.

        :return: The documents of this HyperLinkDraft.
        :rtype: object
        """
        return self._documents

    @documents.setter
    def documents(self, documents):
        """
        Sets the documents of this HyperLinkDraft.
        Detail view of rating.

        :param documents: The documents of this HyperLinkDraft.
        :type: object
        """

        self._documents = documents

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class FailedDependencyError(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, package=None, third_party=None, get_or_purchased=None, to_do=None, to_account=None, dependency_of_package=None, dependency_package_details=None):
        """
        FailedDependencyError - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'package': 'str',
            'third_party': 'bool',
            'get_or_purchased': 'bool',
            'to_do': 'str',
            'to_account': 'str',
            'dependency_of_package': 'str',
            'dependency_package_details': 'object'
        }

        self.attribute_map = {
            'package': 'package',
            'third_party': 'third_party',
            'get_or_purchased': 'get_or_purchased',
            'to_do': 'to_do',
            'to_account': 'to_account',
            'dependency_of_package': 'dependency_of_package',
            'dependency_package_details': 'dependency_package_details'
        }

        self._package = package
        self._third_party = third_party
        self._get_or_purchased = get_or_purchased
        self._to_do = to_do
        self._to_account = to_account
        self._dependency_of_package = dependency_of_package
        self._dependency_package_details = dependency_package_details

    @property
    def package(self):
        """
        Gets the package of this FailedDependencyError.
        fully qualified package name

        :return: The package of this FailedDependencyError.
        :rtype: str
        """
        return self._package

    @package.setter
    def package(self, package):
        """
        Sets the package of this FailedDependencyError.
        fully qualified package name

        :param package: The package of this FailedDependencyError.
        :type: str
        """

        self._package = package

    @property
    def third_party(self):
        """
        Gets the third_party of this FailedDependencyError.
        boolean field to identify third party package.

        :return: The third_party of this FailedDependencyError.
        :rtype: bool
        """
        return self._third_party

    @third_party.setter
    def third_party(self, third_party):
        """
        Sets the third_party of this FailedDependencyError.
        boolean field to identify third party package.

        :param third_party: The third_party of this FailedDependencyError.
        :type: bool
        """

        self._third_party = third_party

    @property
    def get_or_purchased(self):
        """
        Gets the get_or_purchased of this FailedDependencyError.
        boolean field to identify purchased packages.

        :return: The get_or_purchased of this FailedDependencyError.
        :rtype: bool
        """
        return self._get_or_purchased

    @get_or_purchased.setter
    def get_or_purchased(self, get_or_purchased):
        """
        Sets the get_or_purchased of this FailedDependencyError.
        boolean field to identify purchased packages.

        :param get_or_purchased: The get_or_purchased of this FailedDependencyError.
        :type: bool
        """

        self._get_or_purchased = get_or_purchased

    @property
    def to_do(self):
        """
        Gets the to_do of this FailedDependencyError.
        message for end user.

        :return: The to_do of this FailedDependencyError.
        :rtype: str
        """
        return self._to_do

    @to_do.setter
    def to_do(self, to_do):
        """
        Sets the to_do of this FailedDependencyError.
        message for end user.

        :param to_do: The to_do of this FailedDependencyError.
        :type: str
        """

        self._to_do = to_do

    @property
    def to_account(self):
        """
        Gets the to_account of this FailedDependencyError.
        intended recipient during initiated operation.

        :return: The to_account of this FailedDependencyError.
        :rtype: str
        """
        return self._to_account

    @to_account.setter
    def to_account(self, to_account):
        """
        Sets the to_account of this FailedDependencyError.
        intended recipient during initiated operation.

        :param to_account: The to_account of this FailedDependencyError.
        :type: str
        """

        self._to_account = to_account

    @property
    def dependency_of_package(self):
        """
        Gets the dependency_of_package of this FailedDependencyError.
        package for which the package is dependency of.

        :return: The dependency_of_package of this FailedDependencyError.
        :rtype: str
        """
        return self._dependency_of_package

    @dependency_of_package.setter
    def dependency_of_package(self, dependency_of_package):
        """
        Sets the dependency_of_package of this FailedDependencyError.
        package for which the package is dependency of.

        :param dependency_of_package: The dependency_of_package of this FailedDependencyError.
        :type: str
        """

        self._dependency_of_package = dependency_of_package

    @property
    def dependency_package_details(self):
        """
        Gets the dependency_package_details of this FailedDependencyError.
        details of the dependency package.

        :return: The dependency_package_details of this FailedDependencyError.
        :rtype: object
        """
        return self._dependency_package_details

    @dependency_package_details.setter
    def dependency_package_details(self, dependency_package_details):
        """
        Sets the dependency_package_details of this FailedDependencyError.
        details of the dependency package.

        :param dependency_package_details: The dependency_package_details of this FailedDependencyError.
        :type: object
        """

        self._dependency_package_details = dependency_package_details

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class ValidatorSetsInBadge(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, badge_name=None, badge_slug=None, description=None, mandatory_for=None, message=None, modified_date=None, created_date=None, list_of_validator_sets=None):
        """
        ValidatorSetsInBadge - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'badge_name': 'str',
            'badge_slug': 'str',
            'description': 'str',
            'mandatory_for': 'list[str]',
            'message': 'str',
            'modified_date': 'datetime',
            'created_date': 'datetime',
            'list_of_validator_sets': 'list[list[ValidatorDetail]]'
        }

        self.attribute_map = {
            'badge_name': 'badge_name',
            'badge_slug': 'badge_slug',
            'description': 'description',
            'mandatory_for': 'mandatory_for',
            'message': 'message',
            'modified_date': 'modified_date',
            'created_date': 'created_date',
            'list_of_validator_sets': 'list_of_validator_sets'
        }

        self._badge_name = badge_name
        self._badge_slug = badge_slug
        self._description = description
        self._mandatory_for = mandatory_for
        self._message = message
        self._modified_date = modified_date
        self._created_date = created_date
        self._list_of_validator_sets = list_of_validator_sets

    @property
    def badge_name(self):
        """
        Gets the badge_name of this ValidatorSetsInBadge.
        badge name/title

        :return: The badge_name of this ValidatorSetsInBadge.
        :rtype: str
        """
        return self._badge_name

    @badge_name.setter
    def badge_name(self, badge_name):
        """
        Sets the badge_name of this ValidatorSetsInBadge.
        badge name/title

        :param badge_name: The badge_name of this ValidatorSetsInBadge.
        :type: str
        """

        self._badge_name = badge_name

    @property
    def badge_slug(self):
        """
        Gets the badge_slug of this ValidatorSetsInBadge.
        badge slug

        :return: The badge_slug of this ValidatorSetsInBadge.
        :rtype: str
        """
        return self._badge_slug

    @badge_slug.setter
    def badge_slug(self, badge_slug):
        """
        Sets the badge_slug of this ValidatorSetsInBadge.
        badge slug

        :param badge_slug: The badge_slug of this ValidatorSetsInBadge.
        :type: str
        """

        self._badge_slug = badge_slug

    @property
    def description(self):
        """
        Gets the description of this ValidatorSetsInBadge.
        description representing the badge

        :return: The description of this ValidatorSetsInBadge.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this ValidatorSetsInBadge.
        description representing the badge

        :param description: The description of this ValidatorSetsInBadge.
        :type: str
        """

        self._description = description

    @property
    def mandatory_for(self):
        """
        Gets the mandatory_for of this ValidatorSetsInBadge.
        Actions for which badge is mandatory

        :return: The mandatory_for of this ValidatorSetsInBadge.
        :rtype: list[str]
        """
        return self._mandatory_for

    @mandatory_for.setter
    def mandatory_for(self, mandatory_for):
        """
        Sets the mandatory_for of this ValidatorSetsInBadge.
        Actions for which badge is mandatory

        :param mandatory_for: The mandatory_for of this ValidatorSetsInBadge.
        :type: list[str]
        """

        self._mandatory_for = mandatory_for

    @property
    def message(self):
        """
        Gets the message of this ValidatorSetsInBadge.
        message regarding badge

        :return: The message of this ValidatorSetsInBadge.
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """
        Sets the message of this ValidatorSetsInBadge.
        message regarding badge

        :param message: The message of this ValidatorSetsInBadge.
        :type: str
        """

        self._message = message

    @property
    def modified_date(self):
        """
        Gets the modified_date of this ValidatorSetsInBadge.
        last modified date.

        :return: The modified_date of this ValidatorSetsInBadge.
        :rtype: datetime
        """
        return self._modified_date

    @modified_date.setter
    def modified_date(self, modified_date):
        """
        Sets the modified_date of this ValidatorSetsInBadge.
        last modified date.

        :param modified_date: The modified_date of this ValidatorSetsInBadge.
        :type: datetime
        """

        self._modified_date = modified_date

    @property
    def created_date(self):
        """
        Gets the created_date of this ValidatorSetsInBadge.
        package draft creation date.

        :return: The created_date of this ValidatorSetsInBadge.
        :rtype: datetime
        """
        return self._created_date

    @created_date.setter
    def created_date(self, created_date):
        """
        Sets the created_date of this ValidatorSetsInBadge.
        package draft creation date.

        :param created_date: The created_date of this ValidatorSetsInBadge.
        :type: datetime
        """

        self._created_date = created_date

    @property
    def list_of_validator_sets(self):
        """
        Gets the list_of_validator_sets of this ValidatorSetsInBadge.
        List of validator sets

        :return: The list_of_validator_sets of this ValidatorSetsInBadge.
        :rtype: list[list[ValidatorDetail]]
        """
        return self._list_of_validator_sets

    @list_of_validator_sets.setter
    def list_of_validator_sets(self, list_of_validator_sets):
        """
        Sets the list_of_validator_sets of this ValidatorSetsInBadge.
        List of validator sets

        :param list_of_validator_sets: The list_of_validator_sets of this ValidatorSetsInBadge.
        :type: list[list[ValidatorDetail]]
        """

        self._list_of_validator_sets = list_of_validator_sets

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class PublicationValidationError(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, license_name=None, license_type=None, allow_listed=None, price=None, allow_bundled=None, number_of_downstream_packages=None, upgrades=None, label=None, publication_error=None, publication_license_does_not_exist_error=None, not_allow_to_bundle_error=None, license_compatibility_error=None, draft_version_dependency_error=None, version_not_published_error=None, downstream_publication_license_error=None, version_validation_status_error=None):
        """
        PublicationValidationError - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'license_name': 'str',
            'license_type': 'str',
            'allow_listed': 'str',
            'price': 'str',
            'allow_bundled': 'str',
            'number_of_downstream_packages': 'int',
            'upgrades': 'str',
            'label': 'str',
            'publication_error': 'str',
            'publication_license_does_not_exist_error': 'str',
            'not_allow_to_bundle_error': 'str',
            'license_compatibility_error': 'str',
            'draft_version_dependency_error': 'str',
            'version_not_published_error': 'str',
            'downstream_publication_license_error': 'list[str]',
            'version_validation_status_error': 'str'
        }

        self.attribute_map = {
            'license_name': 'license_name',
            'license_type': 'license_type',
            'allow_listed': 'allow_listed',
            'price': 'price',
            'allow_bundled': 'allow_bundled',
            'number_of_downstream_packages': 'number_of_downstream_packages',
            'upgrades': 'upgrades',
            'label': 'label',
            'publication_error': 'publication_error',
            'publication_license_does_not_exist_error': 'publication_license_does_not_exist_error',
            'not_allow_to_bundle_error': 'not_allow_to_bundle_error',
            'license_compatibility_error': 'license_compatibility_error',
            'draft_version_dependency_error': 'draft_version_dependency_error',
            'version_not_published_error': 'version_not_published_error',
            'downstream_publication_license_error': 'downstream_publication_license_error',
            'version_validation_status_error': 'version_validation_status_error'
        }

        self._license_name = license_name
        self._license_type = license_type
        self._allow_listed = allow_listed
        self._price = price
        self._allow_bundled = allow_bundled
        self._number_of_downstream_packages = number_of_downstream_packages
        self._upgrades = upgrades
        self._label = label
        self._publication_error = publication_error
        self._publication_license_does_not_exist_error = publication_license_does_not_exist_error
        self._not_allow_to_bundle_error = not_allow_to_bundle_error
        self._license_compatibility_error = license_compatibility_error
        self._draft_version_dependency_error = draft_version_dependency_error
        self._version_not_published_error = version_not_published_error
        self._downstream_publication_license_error = downstream_publication_license_error
        self._version_validation_status_error = version_validation_status_error

    @property
    def license_name(self):
        """
        Gets the license_name of this PublicationValidationError.
        

        :return: The license_name of this PublicationValidationError.
        :rtype: str
        """
        return self._license_name

    @license_name.setter
    def license_name(self, license_name):
        """
        Sets the license_name of this PublicationValidationError.
        

        :param license_name: The license_name of this PublicationValidationError.
        :type: str
        """

        self._license_name = license_name

    @property
    def license_type(self):
        """
        Gets the license_type of this PublicationValidationError.
        

        :return: The license_type of this PublicationValidationError.
        :rtype: str
        """
        return self._license_type

    @license_type.setter
    def license_type(self, license_type):
        """
        Sets the license_type of this PublicationValidationError.
        

        :param license_type: The license_type of this PublicationValidationError.
        :type: str
        """

        self._license_type = license_type

    @property
    def allow_listed(self):
        """
        Gets the allow_listed of this PublicationValidationError.
        

        :return: The allow_listed of this PublicationValidationError.
        :rtype: str
        """
        return self._allow_listed

    @allow_listed.setter
    def allow_listed(self, allow_listed):
        """
        Sets the allow_listed of this PublicationValidationError.
        

        :param allow_listed: The allow_listed of this PublicationValidationError.
        :type: str
        """

        self._allow_listed = allow_listed

    @property
    def price(self):
        """
        Gets the price of this PublicationValidationError.
        

        :return: The price of this PublicationValidationError.
        :rtype: str
        """
        return self._price

    @price.setter
    def price(self, price):
        """
        Sets the price of this PublicationValidationError.
        

        :param price: The price of this PublicationValidationError.
        :type: str
        """

        self._price = price

    @property
    def allow_bundled(self):
        """
        Gets the allow_bundled of this PublicationValidationError.
        

        :return: The allow_bundled of this PublicationValidationError.
        :rtype: str
        """
        return self._allow_bundled

    @allow_bundled.setter
    def allow_bundled(self, allow_bundled):
        """
        Sets the allow_bundled of this PublicationValidationError.
        

        :param allow_bundled: The allow_bundled of this PublicationValidationError.
        :type: str
        """

        self._allow_bundled = allow_bundled

    @property
    def number_of_downstream_packages(self):
        """
        Gets the number_of_downstream_packages of this PublicationValidationError.


        :return: The number_of_downstream_packages of this PublicationValidationError.
        :rtype: int
        """
        return self._number_of_downstream_packages

    @number_of_downstream_packages.setter
    def number_of_downstream_packages(self, number_of_downstream_packages):
        """
        Sets the number_of_downstream_packages of this PublicationValidationError.


        :param number_of_downstream_packages: The number_of_downstream_packages of this PublicationValidationError.
        :type: int
        """

        self._number_of_downstream_packages = number_of_downstream_packages

    @property
    def upgrades(self):
        """
        Gets the upgrades of this PublicationValidationError.
        

        :return: The upgrades of this PublicationValidationError.
        :rtype: str
        """
        return self._upgrades

    @upgrades.setter
    def upgrades(self, upgrades):
        """
        Sets the upgrades of this PublicationValidationError.
        

        :param upgrades: The upgrades of this PublicationValidationError.
        :type: str
        """

        self._upgrades = upgrades

    @property
    def label(self):
        """
        Gets the label of this PublicationValidationError.
        

        :return: The label of this PublicationValidationError.
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """
        Sets the label of this PublicationValidationError.
        

        :param label: The label of this PublicationValidationError.
        :type: str
        """

        self._label = label

    @property
    def publication_error(self):
        """
        Gets the publication_error of this PublicationValidationError.
        

        :return: The publication_error of this PublicationValidationError.
        :rtype: str
        """
        return self._publication_error

    @publication_error.setter
    def publication_error(self, publication_error):
        """
        Sets the publication_error of this PublicationValidationError.
        

        :param publication_error: The publication_error of this PublicationValidationError.
        :type: str
        """

        self._publication_error = publication_error

    @property
    def publication_license_does_not_exist_error(self):
        """
        Gets the publication_license_does_not_exist_error of this PublicationValidationError.
        

        :return: The publication_license_does_not_exist_error of this PublicationValidationError.
        :rtype: str
        """
        return self._publication_license_does_not_exist_error

    @publication_license_does_not_exist_error.setter
    def publication_license_does_not_exist_error(self, publication_license_does_not_exist_error):
        """
        Sets the publication_license_does_not_exist_error of this PublicationValidationError.
        

        :param publication_license_does_not_exist_error: The publication_license_does_not_exist_error of this PublicationValidationError.
        :type: str
        """

        self._publication_license_does_not_exist_error = publication_license_does_not_exist_error

    @property
    def not_allow_to_bundle_error(self):
        """
        Gets the not_allow_to_bundle_error of this PublicationValidationError.
        

        :return: The not_allow_to_bundle_error of this PublicationValidationError.
        :rtype: str
        """
        return self._not_allow_to_bundle_error

    @not_allow_to_bundle_error.setter
    def not_allow_to_bundle_error(self, not_allow_to_bundle_error):
        """
        Sets the not_allow_to_bundle_error of this PublicationValidationError.
        

        :param not_allow_to_bundle_error: The not_allow_to_bundle_error of this PublicationValidationError.
        :type: str
        """

        self._not_allow_to_bundle_error = not_allow_to_bundle_error

    @property
    def license_compatibility_error(self):
        """
        Gets the license_compatibility_error of this PublicationValidationError.
        

        :return: The license_compatibility_error of this PublicationValidationError.
        :rtype: str
        """
        return self._license_compatibility_error

    @license_compatibility_error.setter
    def license_compatibility_error(self, license_compatibility_error):
        """
        Sets the license_compatibility_error of this PublicationValidationError.
        

        :param license_compatibility_error: The license_compatibility_error of this PublicationValidationError.
        :type: str
        """

        self._license_compatibility_error = license_compatibility_error

    @property
    def draft_version_dependency_error(self):
        """
        Gets the draft_version_dependency_error of this PublicationValidationError.
        

        :return: The draft_version_dependency_error of this PublicationValidationError.
        :rtype: str
        """
        return self._draft_version_dependency_error

    @draft_version_dependency_error.setter
    def draft_version_dependency_error(self, draft_version_dependency_error):
        """
        Sets the draft_version_dependency_error of this PublicationValidationError.
        

        :param draft_version_dependency_error: The draft_version_dependency_error of this PublicationValidationError.
        :type: str
        """

        self._draft_version_dependency_error = draft_version_dependency_error

    @property
    def version_not_published_error(self):
        """
        Gets the version_not_published_error of this PublicationValidationError.
        

        :return: The version_not_published_error of this PublicationValidationError.
        :rtype: str
        """
        return self._version_not_published_error

    @version_not_published_error.setter
    def version_not_published_error(self, version_not_published_error):
        """
        Sets the version_not_published_error of this PublicationValidationError.
        

        :param version_not_published_error: The version_not_published_error of this PublicationValidationError.
        :type: str
        """

        self._version_not_published_error = version_not_published_error

    @property
    def downstream_publication_license_error(self):
        """
        Gets the downstream_publication_license_error of this PublicationValidationError.
        

        :return: The downstream_publication_license_error of this PublicationValidationError.
        :rtype: list[str]
        """
        return self._downstream_publication_license_error

    @downstream_publication_license_error.setter
    def downstream_publication_license_error(self, downstream_publication_license_error):
        """
        Sets the downstream_publication_license_error of this PublicationValidationError.
        

        :param downstream_publication_license_error: The downstream_publication_license_error of this PublicationValidationError.
        :type: list[str]
        """

        self._downstream_publication_license_error = downstream_publication_license_error

    @property
    def version_validation_status_error(self):
        """
        Gets the version_validation_status_error of this PublicationValidationError.
        When version can not be published within a license due to its validation status

        :return: The version_validation_status_error of this PublicationValidationError.
        :rtype: str
        """
        return self._version_validation_status_error

    @version_validation_status_error.setter
    def version_validation_status_error(self, version_validation_status_error):
        """
        Sets the version_validation_status_error of this PublicationValidationError.
        When version can not be published within a license due to its validation status

        :param version_validation_status_error: The version_validation_status_error of this PublicationValidationError.
        :type: str
        """

        self._version_validation_status_error = version_validation_status_error

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Swagger Blended Hub API Explorer

    A collection of Swagger documentation for the Blended Hub APIs.

    OpenAPI spec version: 1.0.0
    Contact: api@blended.co
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class Like(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, pk=None, like=None, user=None):
        """
        Like - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'pk': 'int',
            'like': 'bool',
            'user': 'str'
        }

        self.attribute_map = {
            'pk': 'pk',
            'like': 'like',
            'user': 'user'
        }

        self._pk = pk
        self._like = like
        self._user = user

    @property
    def pk(self):
        """
        Gets the pk of this Like.
        value used to identify like uniquely.

        :return: The pk of this Like.
        :rtype: int
        """
        return self._pk

    @pk.setter
    def pk(self, pk):
        """
        Sets the pk of this Like.
        value used to identify like uniquely.

        :param pk: The pk of this Like.
        :type: int
        """

        self._pk = pk

    @property
    def like(self):
        """
        Gets the like of this Like.
        package has like or not.

        :return: The like of this Like.
        :rtype: bool
        """
        return self._like

    @like.setter
    def like(self, like):
        """
        Sets the like of this Like.
        package has like or not.

        :param like: The like of this Like.
        :type: bool
        """

        self._like = like

    @property
    def user(self):
        """
        Gets the user of this Like.
        user name.

        :return: The user of this Like.
        :rtype: str
        """
        return self._user

    @user.setter
    def user(self, user):
        """
        Sets the user of this Like.
        user name.

        :param user: The user of this Like.
        :type: str
        """

        self._user = user

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other
